@@ -1,7 +1,7 @@
-/* $Id: minissdp.c,v 1.82 2015/12/12 08:19:44 nanard Exp $ */
+/* $Id: minissdp.c,v 1.85 2017/04/21 11:23:43 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2006-2015 Thomas Bernard
+ * (c) 2006-2017 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -17,6 +17,10 @@
 #include <syslog.h>
 
 #include "config.h"
+#if defined(ENABLE_IPV6) && defined(UPNP_STRICT)
+#include <ifaddrs.h>
+#endif /* defined(ENABLE_IPV6) && defined(UPNP_STRICT) */
+
 #include "upnpdescstrings.h"
 #include "miniupnpdpath.h"
 #include "upnphttp.h"
@@ -116,6 +120,29 @@ AddMulticastMembershipIPv6(int s, unsigned int ifindex)
 }
 #endif
 
+
+#if defined(ENABLE_IPV6) && defined(UPNP_STRICT)
+static int get_link_local_addr(unsigned scope_id, struct in6_addr * addr6)
+{
+	struct ifaddrs * ifap;
+	struct ifaddrs * ife;
+	if(getifaddrs(&ifap)<0) {
+		syslog(LOG_ERR, "getifaddrs: %m");
+		return -1;
+	}
+	for(ife = ifap; ife != NULL; ife = ife->ifa_next) {
+		if(ife->ifa_addr == NULL) continue;
+		if(ife->ifa_addr->sa_family != AF_INET6) continue;
+		if(!IN6_IS_ADDR_LINKLOCAL(&(((const struct sockaddr_in6 *)ife->ifa_addr)->sin6_addr))) continue;
+		if(scope_id != if_nametoindex(ife->ifa_name)) continue;
+		memcpy(addr6, &(((const struct sockaddr_in6 *)ife->ifa_addr)->sin6_addr), sizeof(struct in6_addr));
+		break;
+	}
+	freeifaddrs(ifap);
+	return 0;
+}
+#endif /* defined(ENABLE_IPV6) && defined(UPNP_STRICT) */
+
 /* Open and configure the socket listening for
  * SSDP udp packets sent on 239.255.255.250 port 1900
  * SSDP v6 udp packets sent on FF02::C, or FF05::C, port 1900 */
@@ -296,6 +323,9 @@ OpenAndConfSSDPNotifySocketIPv6(unsigned int if_index)
 {
 	int s;
 	unsigned int loop = 0;
+	/* UDA 2.0 : The hop limit of each IP packet for a Site-Local scope
+	 * multicast message SHALL be configurable and SHOULD default to 10 */
+	int hop_limit = 10;
 	struct sockaddr_in6 sockname;
 
 	s = socket(PF_INET6, SOCK_DGRAM, 0);
@@ -316,6 +346,12 @@ OpenAndConfSSDPNotifySocketIPv6(unsigned int if_index)
 		close(s);
 		return -1;
 	}
+	if(setsockopt(s, IPPROTO_IPV6, IPV6_MULTICAST_HOPS, &hop_limit, sizeof(hop_limit)) < 0)
+	{
+		syslog(LOG_ERR, "setsockopt(udp_notify, IPV6_MULTICAST_HOPS): %m");
+		close(s);
+		return -1;
+	}
 
 	/* bind() socket before using sendto() is not mandatory
 	 * (sendto() will implicitly bind the socket when called on
@@ -521,12 +557,12 @@ static struct {
 	{"urn:schemas-upnp-org:service:WANIPConnection:", 1, uuidvalue_wcd},
 #endif /* IGD_V2 */
 	{"urn:schemas-upnp-org:service:WANCommonInterfaceConfig:", 1, uuidvalue_wan},
-#ifndef UPNP_STRICT
+#ifdef ADVERTISE_WANPPPCONN
 	/* We use WAN IP Connection, not PPP connection,
 	 * but buggy control points may try to use WanPPPConnection
 	 * anyway */
 	{"urn:schemas-upnp-org:service:WANPPPConnection:", 1, uuidvalue_wcd},
-#endif /* UPNP_STRICT */
+#endif /* ADVERTISE_WANPPPCONN */
 #ifdef ENABLE_L3F_SERVICE
 	{"urn:schemas-upnp-org:service:Layer3Forwarding:", 1, uuidvalue_igd},
 #endif /* ENABLE_L3F_SERVICE */
@@ -621,10 +657,14 @@ SendSSDPNotifies(int s, const char * host, unsigned short http_port,
 {
 #ifdef ENABLE_IPV6
 	struct sockaddr_storage sockname;
+	/* UDA 1.1 AnnexA and UDA 2.0 only allow/define the use of
+	 * Link-Local and Site-Local multicast scopes */
 	static struct { const char * p1, * p2; } const mcast_addrs[] =
 		{ { LL_SSDP_MCAST_ADDR, "[" LL_SSDP_MCAST_ADDR "]" },	/* Link Local */
 		  { SL_SSDP_MCAST_ADDR, "[" SL_SSDP_MCAST_ADDR "]" },	/* Site Local */
+#ifndef UPNP_STRICT
 		  { GL_SSDP_MCAST_ADDR, "[" GL_SSDP_MCAST_ADDR "]" },	/* Global */
+#endif /* ! UPNP_STRICT */
 		  { NULL, NULL } };
 	int j;
 #else /* ENABLE_IPV6 */
@@ -876,7 +916,26 @@ ProcessSSDPData(int s, const char *bufr, int n,
 				mx_value = atoi(mx);
 				syslog(LOG_DEBUG, "MX: %.*s (value=%d)", mx_len, mx, mx_value);
 			}
-#endif
+#endif /* defined(UPNP_STRICT) || defined(DELAY_MSEARCH_RESPONSE) */
+#if defined(UPNP_STRICT)
+			/* Fix UDA-1.2.10 Man header empty or invalid */
+			else if((i < n - 4) && (strncasecmp(bufr+i, "man:", 3) == 0))
+			{
+				const char * man;
+				int man_len;
+				man = bufr+i+4;
+				man_len = 0;
+				while((*man == ' ' || *man == '\t') && (man < bufr + n))
+					man++;
+				while(man[man_len]!='\r' && man[man_len]!='\n'
+				     && (man + man_len < bufr + n))
+					man_len++;
+				if(strncmp(man, "\"ssdp:discover\"", 15) != 0) {
+					syslog(LOG_INFO, "ignoring SSDP packet MAN empty or invalid header");
+					return;
+				}
+			}
+#endif /* defined(UPNP_STRICT) */
 		}
 #ifdef UPNP_STRICT
 		/* For multicast M-SEARCH requests, if the search request does
@@ -930,10 +989,13 @@ ProcessSSDPData(int s, const char *bufr, int n,
 				/* retrieve the IPv6 address which
 				 * will be used locally to reach sender */
 				memset(&addr6, 0, sizeof(addr6));
-				if(get_src_for_route_to (sender, &addr6, &addr6_len, &index) < 0) {
+				if(IN6_IS_ADDR_LINKLOCAL(&(((struct sockaddr_in6 *)sender)->sin6_addr))) {
+					get_link_local_addr(((struct sockaddr_in6 *)sender)->sin6_scope_id, &addr6);
+				} else if(get_src_for_route_to (sender, &addr6, &addr6_len, &index) < 0) {
 					syslog(LOG_WARNING, "get_src_for_route_to() failed, using %s", ipv6_addr_for_http_with_brackets);
 					announced_host = ipv6_addr_for_http_with_brackets;
-				} else {
+				}
+				if(announced_host == NULL) {
 					if(inet_ntop(AF_INET6, &addr6,
 					             announced_host_buf+1,
 					             sizeof(announced_host_buf) - 2)) {
