@@ -1,7 +1,7 @@
-/* $Id: upnpevents.c,v 1.31 2015/12/12 09:36:22 nanard Exp $ */
+/* $Id: upnpevents.c,v 1.37 2016/02/20 19:10:17 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2008-2015 Thomas Bernard
+ * (c) 2008-2016 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -18,6 +18,12 @@
 #include <arpa/inet.h>
 #include <errno.h>
 #include "config.h"
+#if defined(LIB_UUID)
+/* as found on linux */
+#include <uuid/uuid.h>
+#elif defined(BSD_UUID)
+#include <uuid.h>
+#endif /* LIB_UUID / BSD_UUID */
 #include "upnpevents.h"
 #include "miniupnpdpath.h"
 #include "upnpglobalvars.h"
@@ -108,11 +114,50 @@ newSubscriber(const char * eventurl, const char * callback, int callbacklen)
 	}
 	memcpy(tmp->callback, callback, callbacklen);
 	tmp->callback[callbacklen] = '\0';
+#if defined(LIB_UUID)
+	{
+		uuid_t uuid;
+		uuid_generate(uuid);
+		memcpy(tmp->uuid, "uuid:", 5);
+		uuid_unparse(uuid, tmp->uuid + 5);
+	}
+#elif defined(BSD_UUID)
+	{
+		uuid_t uuid;
+		uint32_t status;
+		uuid_create(&uuid, &status);
+		if(status != uuid_s_ok) {
+			syslog(LOG_ERR, "uuid_create() failed (%u)", status);
+		} else {
+			char * uuid_str;
+			uuid_to_string(&uuid, &uuid_str, &status);
+			if(status != uuid_s_ok) {
+				syslog(LOG_ERR, "uuid_to_string() failed (%u)", status);
+			} else {
+				if(strlen(uuid_str) != 36) {
+					syslog(LOG_ERR, "uuid_to_string() returned %s", uuid_str);
+					status = (uint32_t)-1;
+				} else {
+					memcpy(tmp->uuid, "uuid:", 5);
+					memcpy(tmp->uuid + 5, uuid_str, 36);
+					tmp->uuid[sizeof(tmp->uuid)-1] = '\0';
+				}
+				free(uuid_str);
+			}
+		}
+		if(status != uuid_s_ok) {
+			/* make a dummy uuid */
+			strncpy(tmp->uuid, uuidvalue_igd, sizeof(tmp->uuid));
+			tmp->uuid[sizeof(tmp->uuid)-1] = '\0';
+			snprintf(tmp->uuid+sizeof(tmp->uuid)-5, 5, "%04lx", random() & 0xffff);
+		}
+	}
+#else
 	/* make a dummy uuid */
-	/* TODO: improve that */
 	strncpy(tmp->uuid, uuidvalue_igd, sizeof(tmp->uuid));
 	tmp->uuid[sizeof(tmp->uuid)-1] = '\0';
-	snprintf(tmp->uuid+37, 5, "%04lx", random() & 0xffff);
+	snprintf(tmp->uuid+sizeof(tmp->uuid)-5, 5, "%04lx", random() & 0xffff);
+#endif
 	return tmp;
 }
 
@@ -273,12 +318,15 @@ upnp_event_notify_connect(struct upnp_event_notify * obj)
 	p += 7;	/* http:// */
 #ifdef ENABLE_IPV6
 	if(*p == '[') {	/* ip v6 */
+		obj->addrstr[i++] = '[';
 		p++;
 		obj->ipv6 = 1;
 		while(*p != ']' && i < (sizeof(obj->addrstr)-1))
 			obj->addrstr[i++] = *(p++);
 		if(*p == ']')
 			p++;
+		if(i < (sizeof(obj->addrstr)-1))
+			obj->addrstr[i++] = ']';
 	} else {
 #endif
 		while(*p != '/' && *p != ':' && i < (sizeof(obj->addrstr)-1))
@@ -304,9 +352,16 @@ upnp_event_notify_connect(struct upnp_event_notify * obj)
 	obj->path = p;
 #ifdef ENABLE_IPV6
 	if(obj->ipv6) {
+		char addrstr_tmp[48];
 		struct sockaddr_in6 * sa = (struct sockaddr_in6 *)&addr;
 		sa->sin6_family = AF_INET6;
-		inet_pton(AF_INET6, obj->addrstr, &(sa->sin6_addr));
+		i = (int)strlen(obj->addrstr);
+		if(i > 2) {
+			i -= 2;
+			memcpy(addrstr_tmp, obj->addrstr + 1, i);
+			addrstr_tmp[i] = '\0';
+			inet_pton(AF_INET6, addrstr_tmp, &(sa->sin6_addr));
+		}
 		sa->sin6_port = htons(port);
 		addrlen = sizeof(struct sockaddr_in6);
 	} else {
@@ -340,7 +395,11 @@ static void upnp_event_prepare(struct upnp_event_notify * obj)
 	static const char notifymsg[] =
 		"NOTIFY %s HTTP/1.1\r\n"
 		"Host: %s%s\r\n"
-		"Content-Type: text/xml\r\n"
+#if (UPNP_VERSION_MAJOR == 1) && (UPNP_VERSION_MINOR == 0)
+		"Content-Type: text/xml\r\n"	/* UDA v1.0 */
+#else
+		"Content-Type: text/xml; charset=\"utf-8\"\r\n"	/* UDA v1.1 or later */
+#endif
 		"Content-Length: %d\r\n"
 		"NT: upnp:event\r\n"
 		"NTS: upnp:propchange\r\n"
